@@ -6,10 +6,12 @@ use std::{
     time::{Duration, SystemTime, UNIX_EPOCH},
 };
 
-use ffi::{self, require_gpgme_ver};
+use cstr_argument::CStrArgument;
+use ffi;
 
 use crate::{
-    notation::SignatureNotations, Error, KeyAlgorithm, KeyListMode, NonNull, Protocol, Validity,
+    notation::SignatureNotations, Context, Error, KeyAlgorithm, KeyListMode, NonNull, Protocol,
+    Result, Validity,
 };
 
 /// Upstream documentation:
@@ -205,18 +207,12 @@ impl Key {
 
     #[inline]
     pub fn fingerprint_raw(&self) -> Option<&CStr> {
-        require_gpgme_ver! {
-            (1, 7) => {
-                unsafe {
-                    (*self.as_raw())
-                        .fpr
-                        .as_ref()
-                        .map(|s| CStr::from_ptr(s))
-                        .or_else(|| self.primary_key()?.fingerprint_raw())
-                }
-            } else {
-                self.primary_key()?.fingerprint_raw()
-            }
+        unsafe {
+            (*self.as_raw())
+                .fpr
+                .as_ref()
+                .map(|s| CStr::from_ptr(s))
+                .or_else(|| self.primary_key()?.fingerprint_raw())
         }
     }
 
@@ -252,17 +248,23 @@ impl Key {
     }
 
     #[inline]
-    pub fn update(&mut self) -> crate::Result<()> {
+    pub fn update(&mut self) -> Result<()> {
         *self = self.updated()?;
         Ok(())
     }
 
     #[inline]
-    pub fn updated(&self) -> crate::Result<Key> {
-        let mut ctx = crate::Context::from_protocol(self.protocol())?;
+    pub fn updated(&self) -> Result<Key> {
+        let mut ctx = Context::from_protocol(self.protocol())?;
         let _ = ctx.set_key_list_mode(self.key_list_mode());
         ctx.refresh_key(self)
     }
+
+    #[inline]
+    pub fn add_uid(&self, userid: impl CStrArgument) -> Result<()> {
+        let mut ctx = Context::from_protocol(self.protocol())?;
+        ctx.add_uid(self, userid)
+    }
 }
 
 impl fmt::Debug for Key {
@@ -420,7 +422,7 @@ impl<'key> Subkey<'key> {
 
     /// Upstream documentation: [`gpgme_pubkey_algo_string`](https://www.gnupg.org/documentation/manuals/gpgme/Public-Key-Algorithms.html#index-gpgme_005fpubkey_005falgo_005fstring)
     #[inline]
-    pub fn algorithm_name(&self) -> crate::Result<String> {
+    pub fn algorithm_name(&self) -> Result<String> {
         unsafe {
             match ffi::gpgme_pubkey_algo_string(self.as_raw()).as_mut() {
                 Some(raw) => {
@@ -444,13 +446,7 @@ impl<'key> Subkey<'key> {
 
     #[inline]
     pub fn keygrip_raw(&self) -> Option<&'key CStr> {
-        require_gpgme_ver! {
-            (1, 7) => {
-                unsafe { (*self.as_raw()).keygrip.as_ref().map(|s| CStr::from_ptr(s)) }
-            } else {
-                None
-            }
-        }
+        unsafe { (*self.as_raw()).keygrip.as_ref().map(|s| CStr::from_ptr(s)) }
     }
 
     #[inline]
@@ -482,13 +478,7 @@ impl<'key> Subkey<'key> {
 
     #[inline]
     pub fn curve_raw(&self) -> Option<&'key CStr> {
-        require_gpgme_ver! {
-            (1,5) => {
-                unsafe { (*self.as_raw()).curve.as_ref().map(|s| CStr::from_ptr(s)) }
-            } else {
-                None
-            }
-        }
+        unsafe { (*self.as_raw()).curve.as_ref().map(|s| CStr::from_ptr(s)) }
     }
 }
 
@@ -574,20 +564,16 @@ impl<'key> UserId<'key> {
     }
 
     #[inline]
+    #[cfg(feature = "v1_14")]
     pub fn uidhash(&self) -> Result<&'key str, Option<Utf8Error>> {
         self.uidhash_raw()
             .map_or(Err(None), |s| s.to_str().map_err(Some))
     }
 
     #[inline]
+    #[cfg(feature = "v1_14")]
     pub fn uidhash_raw(&self) -> Option<&'key CStr> {
-        require_gpgme_ver! {
-            (1, 14) => {
-                unsafe { (*self.as_raw()).uidhash.as_ref().map(|s| CStr::from_ptr(s)) }
-            } else {
-                None
-            }
-        }
+        unsafe { (*self.as_raw()).uidhash.as_ref().map(|s| CStr::from_ptr(s)) }
     }
 
     #[inline]
@@ -626,14 +612,10 @@ impl<'key> UserId<'key> {
         unsafe { crate::KeyOrigin::from_raw((*self.as_raw()).origin()) }
     }
 
-    require_gpgme_ver! {
-        (1, 8) => {
-            #[inline]
-            pub fn last_update(&self) -> SystemTime {
-                let timestamp = unsafe { (*self.as_raw()).last_update };
-                UNIX_EPOCH + Duration::from_secs(timestamp.into())
-            }
-        }
+    #[inline]
+    pub fn last_update(&self) -> SystemTime {
+        let timestamp = unsafe { (*self.as_raw()).last_update };
+        UNIX_EPOCH + Duration::from_secs(timestamp.into())
     }
 
     #[inline]
@@ -658,17 +640,11 @@ impl<'key> UserId<'key> {
 
     #[inline]
     pub fn tofu_info(&self) -> Option<crate::TofuInfo<'key>> {
-        require_gpgme_ver! {
-            (1,7) => {
-                unsafe {
-                    (*self.as_raw())
-                        .tofu
-                        .as_mut()
-                        .map(|t| crate::TofuInfo::from_raw(t))
-                }
-            } else {
-                None
-            }
+        unsafe {
+            (*self.as_raw())
+                .tofu
+                .as_mut()
+                .map(|t| crate::TofuInfo::from_raw(t))
         }
     }
 }
@@ -879,45 +855,46 @@ impl<'key> UserIdSignature<'key> {
         unsafe { SignatureNotations::from_list((*self.as_raw()).notations) }
     }
 
-    require_gpgme_ver! {
-        (1, 16) => {
-            #[inline]
-            pub fn is_trust_signature(&self) -> bool {
-                self.trust_depth() != 0
-            }
+    #[inline]
+    #[cfg(feature = "v1_16")]
+    pub fn is_trust_signature(&self) -> bool {
+        self.trust_depth() != 0
+    }
 
-            #[inline]
-            pub fn trust_value(&self) -> SignatureTrust {
-                let value = unsafe {
-                    (*self.as_raw()).trust_value()
-                };
-                if !self.is_trust_signature() {
-                    SignatureTrust::None
-                } else if value >= 120 {
-                    SignatureTrust::Complete
-                } else {
-                    SignatureTrust::Partial
-                }
-            }
+    #[inline]
+    #[cfg(feature = "v1_16")]
+    pub fn trust_value(&self) -> SignatureTrust {
+        let value = unsafe { (*self.as_raw()).trust_value() };
+        if !self.is_trust_signature() {
+            SignatureTrust::None
+        } else if value >= 120 {
+            SignatureTrust::Complete
+        } else {
+            SignatureTrust::Partial
+        }
+    }
 
-            #[inline]
-            pub fn trust_depth(&self) -> u8 {
-                unsafe {
-                    (*self.as_raw()).trust_depth()
-                }
-            }
+    #[inline]
+    #[cfg(feature = "v1_16")]
+    pub fn trust_depth(&self) -> u8 {
+        unsafe { (*self.as_raw()).trust_depth() }
+    }
 
-            #[inline]
-            pub fn trust_scope(&self) -> Result<&'key str, Option<Utf8Error>> {
-                self.trust_scope_raw().map_or(Err(None), |s| s.to_str().map_err(Some))
-            }
+    #[inline]
+    #[cfg(feature = "v1_16")]
+    pub fn trust_scope(&self) -> Result<&'key str, Option<Utf8Error>> {
+        self.trust_scope_raw()
+            .map_or(Err(None), |s| s.to_str().map_err(Some))
+    }
 
-            #[inline]
-            pub fn trust_scope_raw(&self) -> Option<&'key CStr> {
-                unsafe {
-                    (*self.as_raw()).trust_scope.as_ref().map(|s| CStr::from_ptr(s))
-                }
-            }
+    #[inline]
+    #[cfg(feature = "v1_16")]
+    pub fn trust_scope_raw(&self) -> Option<&'key CStr> {
+        unsafe {
+            (*self.as_raw())
+                .trust_scope
+                .as_ref()
+                .map(|s| CStr::from_ptr(s))
         }
     }
 }
